@@ -8,7 +8,7 @@ use Mouse;
 use Lemonldap::NG::Portal::Main::Constants
   qw(PE_OK PE_PP_PASSWORD_EXPIRED PE_PP_CHANGE_AFTER_RESET);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Portal::Auth::LDAP';
 
@@ -19,7 +19,7 @@ has adPwdMaxAge => (
     lazy    => 1,
     builder => sub {
         my $conf = $_[0]->{conf};
-        my $res = $conf->{ADPwdMaxAge} || 0;
+        my $res  = $conf->{ADPwdMaxAge} || 0;
         return $res * 10000000;   # padding with '0' to obtain 0.1 micro-seconds
     }
 );
@@ -29,7 +29,7 @@ has adPwdExpireWarning => (
     lazy    => 1,
     builder => sub {
         my $conf = $_[0]->{conf};
-        my $res = $conf->{ADPwdExpireWarning} || 0;
+        my $res  = $conf->{ADPwdExpireWarning} || 0;
         return $res * 10000000;   # padding with '0' to obtain 0.1 micro-seconds
     }
 );
@@ -66,11 +66,16 @@ sub authenticate {
     my ( $self, $req ) = @_;
     my $res = $self->SUPER::authenticate($req);
 
+    my $pls = $self->ldap->getLdapValue( $req->data->{ldapentry}, 'pwdLastSet' );
+    my $computed = $self->ldap->getLdapValue( $req->data->{ldapentry},
+        'msDS-User-Account-Control-Computed' );
+    my $_adUac =
+      $self->ldap->getLdapValue( $req->data->{ldapentry}, 'userAccountControl' )
+      || 0;
+
     unless ( $res == PE_OK ) {
 
         # Check specific AD attributes
-        my $pls      = $req->{sessionInfo}->{_AD_pwdLastSet};
-        my $computed = $req->{sessionInfo}->{_AD_msDS_UACC};
         my $mask = 0xf00000;    # mask to get the 8 at 6th position
         my $expired_flag =
           0x800000;   # 8 at 6th position for flag UF_PASSWORD_EXPIRED to be set
@@ -88,14 +93,10 @@ sub authenticate {
 
     }
     else {
-
-        # get userAccountControl to ckeck password expiration flags
-        my $_adUac = $req->{sessionInfo}->{_AD_userAccountControl} || 0;
-
         my $timestamp = $self->adTime;
 
         # Compute password expiration time (date)
-        my $_pwdExpire = $req->{sessionInfo}->{_AD_pwdLastSet} || $timestamp;
+        my $_pwdExpire = $pls || $timestamp;
         $_pwdExpire += $self->adPwdMaxAge;
 
         # computing when the warning message is displayed on portal
@@ -113,6 +114,7 @@ sub authenticate {
             my $remainingTime = $_pwdExpire - $timestamp;
             $req->info(
                 $self->loadTemplate(
+                    $req,
                     'pwdWillExpire',
                     params => {
                         time => join(
@@ -132,12 +134,16 @@ sub authenticate {
     }
 
     # Remember password if password reset needed
-    $req->data->{oldpassword} = $req->data->{password}
-      if (
+    if (
         $res == PE_PP_CHANGE_AFTER_RESET
         or (    $res == PE_PP_PASSWORD_EXPIRED
             and $self->conf->{ldapAllowResetExpiredPassword} )
-      );
+      )
+    {
+        $req->data->{oldpassword} = $self->{password};
+        $req->data->{noerror}     = 1;
+        $self->setSecurity($req);
+    }
 
     return $res;
 }
