@@ -12,14 +12,14 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_SENDRESPONSE
 );
 
-our $VERSION = '2.0.4';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Portal::Main::Auth', 'Lemonldap::NG::Portal::Lib::CAS';
 
 # PROPERTIES
 
 has srvNumber => ( is => 'rw', default => 0 );
-has srvList => ( is => 'rw', default => sub { [] } );
+has srvList   => ( is => 'rw', default => sub { [] } );
 use constant sessionKind => 'CAS';
 
 # INITIALIZATION
@@ -83,6 +83,9 @@ sub extractFormInfo {
     # Local URL
     my $local_url = $self->p->fullUrl($req);
 
+    # Remove cancel parameter
+    $local_url =~ s/cancel=1&?//;
+
     # Catch proxy callback
     if ( $req->param('casProxy') ) {
         $self->logger->debug("CAS: Proxy callback detected");
@@ -224,7 +227,7 @@ sub extractFormInfo {
         # Get a proxy ticket for each proxied service
         foreach ( keys %$proxied ) {
             my $service = $proxied->{$_};
-            my $pt = $self->retrievePT( $service, $pgtId, $srvConf );
+            my $pt      = $self->retrievePT( $service, $pgtId, $srvConf );
 
             unless ($pt) {
                 $self->logger->error(
