@@ -3,10 +3,12 @@ package Lemonldap::NG::Portal::Lib::LDAP;
 use strict;
 use Mouse;
 use Lemonldap::NG::Portal::Lib::Net::LDAP;
+use Lemonldap::NG::Portal::Main::Constants
+  qw(PE_OK PE_LDAPCONNECTFAILED PE_LDAPERROR PE_BADCREDENTIALS);
 
 extends 'Lemonldap::NG::Common::Module';
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.6';
 
 # PROPERTIES
 
@@ -16,6 +18,17 @@ has ldap => (
     builder => 'newLdap',
 );
 
+has attrs => (
+    is      => 'rw',
+    lazy    => 1,
+    builder => sub {
+        return [
+            values %{ $_[0]->{conf}->{exportedVars} },
+            values %{ $_[0]->{conf}->{ldapExportedVars} }
+        ];
+    }
+);
+
 sub newLdap {
     my $self = $_[0];
     my $ldap;
@@ -98,6 +111,38 @@ sub init {
 
 # RUNNING METHODS
 
+sub getUser {
+    my ( $self, $req, %args ) = @_;
+    return PE_LDAPCONNECTFAILED unless $self->ldap and $self->bind();
+    my $mesg = $self->ldap->search(
+        base   => $self->conf->{ldapBase},
+        scope  => 'sub',
+        filter => (
+              $args{useMail}
+            ? $self->mailFilter->($req)
+            : $self->filter->($req)
+        ),
+        defer => $self->conf->{ldapSearchDeref} || 'find',
+        attrs => $self->attrs,
+    );
+    if ( $mesg->code() != 0 ) {
+        $self->logger->error( 'LDAP Search error: ' . $mesg->error );
+        return PE_LDAPERROR;
+    }
+    if ( $mesg->count() > 1 ) {
+        $self->logger->error('More than one entry returned by LDAP directory');
+        eval { $self->p->_authentication->setSecurity($req) };
+        return PE_BADCREDENTIALS;
+    }
+    unless ( $req->data->{ldapentry} = $mesg->entry(0) ) {
+        $self->userLogger->warn("$req->{user} was not found in LDAP directory");
+        eval { $self->p->_authentication->setSecurity($req) };
+        return PE_BADCREDENTIALS;
+    }
+    $req->data->{dn} = $req->data->{ldapentry}->dn();
+    PE_OK;
+}
+
 # Test LDAP connection before trying to bind
 sub bind {
     my $self = shift;
