@@ -4,7 +4,7 @@ use strict;
 use Mouse;
 use JSON qw(from_json);
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.6';
 
 no warnings 'redefine';
 
@@ -35,10 +35,10 @@ sub checkForNotifications {
 
     # Transform notifications
     my $i = 0;    #Files count
+    my @res;
     foreach my $file ( values %$notifs ) {
-        my $json = from_json($file);
-        my $j    = 0;                  #Notifications count
-        my @res;
+        my $json = from_json( $file, { allow_nonref => 1 } );
+        my $j = 0;    #Notifications count
         $json = [$json] unless ( ref $json eq 'ARRAY' );
       LOOP: foreach my $notif ( @{$json} ) {
 
@@ -62,15 +62,15 @@ sub checkForNotifications {
         # Go to next file if no notification found
         next unless $j;
         $i++;
-        $form .= $self->toForm(@res);
     }
+    @res =
+      sort { $a->{date} cmp $b->{date} or $a->{reference} cmp $b->{reference} }
+      @res;
+    $form .= $self->toForm( $req, @res );
 
     # Stop here if nothing to display
     return 0 unless $i;
 
-    # Cipher cookie
-    $req->id( $self->p->HANDLER->tsv->{cipher}->encrypt( $req->id ) );
-
     # Returns HTML fragment
     return $form;
 }
@@ -78,26 +78,46 @@ sub checkForNotifications {
 sub getNotifBack {
     my ( $self, $req, $name ) = @_;
 
-    # Look if all notifications have been accepted. If not, redirects to
-    # portal
-
     # Search for Lemonldap::NG cookie (ciphered)
     my $id;
     unless ( $id = $req->cookies->{ $self->{conf}->{cookieName} } ) {
         return $self->p->sendError( $req, 'No cookie found', 401 );
     }
+
+    if ( $req->param('cancel') ) {
+        $self->logger->debug('Cancel called -> remove ciphered cookie');
+        $req->addCookie(
+            $self->p->cookie(
+                name    => $self->conf->{cookieName},
+                value   => 0,
+                domain  => $self->conf->{domain},
+                secure  => 0,
+                expires => 'Wed, 21 Oct 2015 00:00:00 GMT'
+            )
+        );
+        $req->mustRedirect(1);
+        return $self->p->do( $req, [] );
+    }
+
+    # Look if all notifications have been accepted. If not, redirect to
+    # portal
+
+    # Try to decrypt Lemonldap::NG ciphered cookie
     $id = $self->p->HANDLER->tsv->{cipher}->decrypt($id)
-      or return $self->sendError( $req, 'Unable to decrypt', 500 );
+      or return $self->p->sendError( $req, 'Unable to decrypt', 400 );
 
-    # Verify that session exists
-    $req->userData( $self->p->HANDLER->retrieveSession($id) )
-      or return $self->sendError( $req, 'Unknown session', 401 );
+    # Check that session exists
+    $req->userData( $self->p->HANDLER->retrieveSession( $req, $id ) )
+      or return $self->p->sendError( $req, 'Unknown session', 401 );
 
-    # Restore datas
-    $self->p->importHandlerDatas($req);
+    # Restore data
+    $self->p->importHandlerData($req);
     my $uid = $req->sessionInfo->{ $self->notifObject->notifField };
 
-    my ( $notifs, $forUser ) = $self->notifObject->getNotifications($uid);
+    my ( $notifs, $forUser );
+    eval { ( $notifs, $forUser ) = $self->notifObject->getNotifications($uid) };
+    return $self->p->sendError( $req, $@, 500 ) if ($@);
+
     if ($notifs) {
 
         # Get accepted notifications
@@ -117,7 +137,7 @@ sub getNotifBack {
         foreach my $fileName ( keys %$notifs ) {
             my $file       = $notifs->{$fileName};
             my $fileResult = 1;
-            my $json       = from_json($file);
+            my $json       = from_json( $file, { allow_nonref => 1 } );
             $json = [$json] unless ( ref $json eq 'ARRAY' );
 
             # Get pending notifications and verify that they have been accepted
@@ -157,8 +177,7 @@ sub getNotifBack {
                 $self->p->updatePersistentSession( $req,
                     { "notification_$reference" => time() } );
                 $self->logger->debug(
-                    "Notification $reference registered in persistent session"
-                );
+                    "Notification $reference registered in persistent session");
             }
 
         # Notifications accepted for this file, delete it unless it's a wildcard
@@ -171,17 +190,17 @@ sub getNotifBack {
 
             # One pending notification has been found and not accepted,
             # restart process to display pending notifications
-            # TODO: is it a good idea to launch all 'afterDatas' subs ?
+            # TODO: is it a good idea to launch all 'endAuth' subs ?
             $self->logger->debug(
                 'Pending notification has been found and not accepted');
-            return $self->p->do( $req, $self->p->afterDatas );
+            return $self->p->do( $req, [ @{ $self->p->endAuth } ] );
         }
 
         # All pending notifications have been accepted, restore cookies and
         # launch 'controlUrl' to restore "urldc" using do()
         $self->logger->debug('All pending notifications have been accepted');
         $self->p->rebuildCookies($req);
-        return $self->p->do( $req, ['controlUrl'] );
+        return $self->p->do( $req, [ 'controlUrl', @{ $self->p->endAuth } ] );
     }
     else {
         # No notifications checked here, this entry point must not be called.
@@ -193,7 +212,7 @@ sub getNotifBack {
 }
 
 sub toForm {
-    my ( $self, @notifs ) = @_;
+    my ( $self, $req, @notifs ) = @_;
     my $i = 0;
     @notifs = map {
         $i++;
@@ -206,30 +225,72 @@ sub toForm {
         $_->{id} = "1x$i";
         $_;
     } @notifs;
-    require HTML::Template;
-    my $file =
-        $self->conf->{templateDir} . '/'
-      . $self->conf->{portalSkin}
-      . '/notifinclude.tpl';
-    $file = $self->conf->{templateDir} . '/common/notifinclude.tpl'
-      unless ( -e $file );
-    my $tpl = HTML::Template->new(
-        filename               => $file,
-        die_on_bad_params      => 0,
-        die_on_missing_include => 1,
-        cache                  => 1,
-        global_vars            => 0,
-    );
-    $tpl->param( notifications => \@notifs );
-    return $tpl->output;
+    return $self->loadTemplate( $req, 'notifinclude',
+        params => { notifications => \@notifs } );
 }
 
 sub notificationServer {
-    my ( $self, $req ) = @_;
+    my ( $self, $req, @args ) = @_;
+    $self->p->logger->debug("REST request for notifications");
     return $self->p->sendError( $req, 'Only JSON requests here', 400 )
       unless ( $req->wantJSON );
-    my $res = $self->notifObject->newNotification( $req->content );
-    return $self->p->sendError( $req, 'Bad request', 400 ) unless ($res);
+    $self->p->logger->debug( "JSON content: " . $req->content );
+    my ( $res, $err );
+    if ( $req->method =~ /^POST$/i ) {
+        $self->p->logger->debug("POST request");
+        ( $res, $err ) =
+          eval { $self->notifObject->newNotification( $req->content ) };
+        return $self->p->sendError( $req, $@, 500 ) if ($@);
+    }
+    elsif ( $req->method =~ /^GET$/i ) {
+        $self->p->logger->debug("GET request");
+        my ( $uid, $ref ) = @args;
+        my $notifs;
+        ( $notifs, $err ) =
+          eval { $self->notifObject->getNotifications($uid) };
+        return $self->p->sendError( $req, $@, 500 ) if ($@);
+        $res = [];
+        foreach my $notif ( keys %$notifs ) {
+            $self->p->logger->debug("Found notification $notif");
+            my $json;
+            eval {
+                $json = from_json( $notifs->{$notif}, { allow_nonref => 1 } );
+            };
+            return $self->p->sendError( $req, "Unable to decode JSON file: $@",
+                400 )
+              if ($@);
+            $self->p->logger->debug(
+                "Notification $notif: " . $notifs->{$notif} );
+            if ($ref) {
+                push( @$res,
+                    map { "$_" => $json->{$_} },
+                    split /\s+/,
+                    $self->conf->{notificationServerSentAttributes} )
+                  if ( $json->{reference} =~ /^$ref$/ );
+            }
+            else {
+                push @$res,
+                  { "uid" => $json->{uid}, "reference" => $json->{reference} };
+            }
+        }
+    }
+    elsif ( $req->method =~ /^DELETE$/i ) {
+        $self->p->logger->debug("DELETE request");
+        my $uid = $req->params('uid');
+        my $ref = $req->params('reference');
+        return $self->p->sendError( $req,
+            "Missing parameters -> uid: $uid / ref: $ref", 400 )
+          unless ( $uid and $ref );
+        ( $res, $err ) =
+          eval { $self->notifObject->deleteNotification( $uid, $ref ); };
+        return $self->p->sendError( $req, $@, 500 ) if ($@);
+    }
+    else {
+        return $self->p->sendError( $req, "Unknown HTTP method: $req->method",
+            400 );
+    }
+    return $self->p->sendError( $req, ( $err ? $err : 'Bad request' ), 400 )
+      unless ($res);
     return $self->p->sendJSONresponse( $req, { result => $res } );
 }
 
