@@ -8,7 +8,7 @@
 #                  of lemonldap-ng.ini) and underlying handler configuration
 package Lemonldap::NG::Portal::Main::Init;
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.6';
 
 package Lemonldap::NG::Portal::Main;
 
@@ -38,7 +38,8 @@ has _groups     => ( is => 'rw' );
 has _jsRedirect => ( is => 'rw' );
 
 # TrustedDomain regexp
-has trustedDomainsRe => ( is => 'rw' );
+has trustedDomainsRe         => ( is => 'rw' );
+has additionalTrustedDomains => ( is => 'rw', default => sub { [] } );
 
 # Lists to store plugins entry-points
 my @entryPoints;
@@ -80,9 +81,12 @@ has spRules => (
 # Custom template parameters
 has customParameters => ( is => 'rw', default => sub { {} } );
 
-# Content-Security-Policy header
+# Content-Security-Policy headers
 has csp => ( is => 'rw' );
 
+# Cross-Origine Resource Sharing headers
+has cors => ( is => 'rw' );
+
 # INITIALIZATION
 
 sub init {
@@ -153,6 +157,9 @@ sub init {
       # Refresh session
       ->addAuthRoute( refresh => 'refresh', ['GET'] )
 
+      ->addAuthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
+      ->addUnauthRoute( '*' => 'corsPreflight', ['OPTIONS'] )
+
       # Logout
       ->addAuthRoute( logout => 'logout', ['GET'] );
 
@@ -179,13 +186,29 @@ sub reloadConf {
         $self->{conf}->{$key} ||= $conf->{$key};
     }
 
-    # Initialize content-security-policy header
+    # Initialize content-security-policy headers
     my $csp = '';
     foreach (qw(default img src style font connect script)) {
         my $prm = $self->conf->{ 'csp' . ucfirst($_) };
         $csp .= "$_-src $prm;" if ($prm);
     }
     $self->csp($csp);
+    $self->logger->debug( "Initialized CSP headers : " . $self->csp );
+
+    # Initialize Cross-Origin Resource Sharing headers
+    my $cors = '';
+    foreach (
+        qw(Allow_Origin Allow_Credentials Allow_Headers Allow_Methods Expose_Headers Max_Age)
+      )
+    {
+        my $header = $_;
+        my $prm    = $self->conf->{ 'cors' . $_ };
+        $header =~ s/_/-/;
+        $prm    =~ s/\s+//;
+        $cors .= "Access-Control-$header;$prm;";
+    }
+    $self->cors($cors);
+    $self->logger->debug( "Initialized CORS headers : " . $self->cors );
 
     # Initialize templateDir
     $self->{templateDir} =
@@ -250,53 +273,6 @@ sub reloadConf {
       unless $self->{_sfEngine} =
       $self->loadPlugin( $self->conf->{'sfEngine'} );
 
-    # Initialize trusted domain regexp
-    if (    $self->conf->{trustedDomains}
-        and $self->conf->{trustedDomains} =~ /^\s*\*\s*$/ )
-    {
-        $self->trustedDomainsRe(qr#^https?://#);
-    }
-    else {
-        my $re = Regexp::Assemble->new();
-        if ( my $td = $self->conf->{trustedDomains} ) {
-            $td =~ s/^\s*(.*?)\s*/$1/;
-            foreach ( split( /\s+/, $td ) ) {
-                next unless ($td);
-                s#^\.#([^/]+\.)?#;
-                $self->logger->debug("Domain $_ added in trusted domains");
-                s/\./\\./g;
-
-                # This regexp is valid for the followings hosts:
-                #  - $td
-                #  - $domainlabel.$td
-                # $domainlabel is build looking RFC2396
-                # (see Regexp::Common::URI::RFC2396)
-                $_ =~
-                  s/\*\\\./(?:(?:[a-zA-Z0-9][-a-zA-Z0-9]*)?[a-zA-Z0-9]\\.)*/g;
-                $re->add("$_");
-            }
-        }
-        my $p = $self->conf->{portal};
-        $p =~ s#https?://([^/]*).*$#$1#;
-        $re->add( quotemeta($p) );
-        foreach my $vhost ( keys %{ $self->conf->{locationRules} } ) {
-            $self->logger->debug("Vhost $vhost added in trusted domains");
-            $re->add( quotemeta($vhost) );
-            $self->conf->{vhostOptions} ||= {};
-            if ( my $tmp =
-                $self->conf->{vhostOptions}->{$vhost}->{vhostAliases} )
-            {
-                foreach my $alias ( split /\s+/, $tmp ) {
-                    $self->logger->debug(
-                        "Alias $alias added in trusted domains");
-                    $re->add( quotemeta($alias) );
-                }
-            }
-        }
-        my $tmp = 'https?://' . $re->as_string . '(?::\d+)?(?:/|$)';
-        $self->trustedDomainsRe(qr/$tmp/);
-    }
-
     # Compile macros in _macros, groups in _groups
     foreach my $type (qw(macros groups)) {
         $self->{"_$type"} = {};
@@ -325,11 +301,66 @@ sub reloadConf {
         $self->loadPlugin($plugin) or return $self->fail;
     }
 
+    # Initialize trusted domain regexp
+    if (    $self->conf->{trustedDomains}
+        and $self->conf->{trustedDomains} =~ /^\s*\*\s*$/ )
+    {
+        $self->trustedDomainsRe(qr#^https?://#);
+    }
+    else {
+        my $re = Regexp::Assemble->new();
+        if ( my $td = $self->conf->{trustedDomains} ) {
+            $td =~ s/^\s*(.*?)\s*/$1/;
+            foreach ( split( /\s+/, $td ) ) {
+                next unless ($td);
+                s#^\.#([^/]+\.)?#;
+                $self->logger->debug("Domain $_ added in trusted domains");
+                s/\./\\./g;
+
+                # This regexp is valid for the followings hosts:
+                #  - $td
+                #  - $domainlabel.$td
+                # $domainlabel is build looking RFC2396
+                # (see Regexp::Common::URI::RFC2396)
+                $_ =~
+                  s/\*\\\./(?:(?:[a-zA-Z0-9][-a-zA-Z0-9]*)?[a-zA-Z0-9]\\.)*/g;
+                $re->add("$_");
+            }
+        }
+        foreach ( @{ $self->{additionalTrustedDomains} },
+            $self->conf->{portal} )
+        {
+            my $p = $_;
+            $p =~ s#https?://([^/]*).*$#$1#;
+            $re->add( quotemeta($p) );
+        }
+        foreach my $vhost ( keys %{ $self->conf->{locationRules} } ) {
+            $self->logger->debug("Vhost $vhost added in trusted domains");
+            $re->add( quotemeta($vhost) );
+            $self->conf->{vhostOptions} ||= {};
+            if ( my $tmp =
+                $self->conf->{vhostOptions}->{$vhost}->{vhostAliases} )
+            {
+                foreach my $alias ( split /\s+/, $tmp ) {
+                    $self->logger->debug(
+                        "Alias $alias added in trusted domains");
+                    $re->add( quotemeta($alias) );
+                }
+            }
+        }
+
+        my $tmp = 'https?://' . $re->as_string . '(?::\d+)?(?:/|$)';
+        $self->trustedDomainsRe(qr/$tmp/);
+
+    }
+
     # Clean $req->pdata after authentication
     push @{ $self->endAuth }, sub {
         unless ( $_[0]->pdata->{keepPdata} ) {
             $self->logger->debug('Cleaning pdata');
             $_[0]->pdata( {} );
+            $self->userLogger->notice( $_[0]->user . ' connected' )
+              if $_[0]->user;
         }
         return PE_OK;
     };
@@ -408,7 +439,7 @@ sub findEP {
         }
         else {
             foreach my $ep ( keys %$h ) {
-                my $callback = $h->{$ep};
+                my $callback    = $h->{$ep};
                 my $previousSub = $self->aroundSub->{$ep} ||= sub {
                     $self->logger->debug(
                         "$ep launched inside ${plugin}::$callback");
