@@ -5,8 +5,13 @@ package Lemonldap::NG::Portal::Main::Plugin;
 use strict;
 use Mouse;
 use HTML::Template;
+use Lemonldap::NG::Portal::Main::Constants qw(
+  PE_OK
+  PE_INFO
+  PE_ERROR
+);
 
-our $VERSION = '2.0.0';
+our $VERSION = '2.0.6';
 
 extends 'Lemonldap::NG::Common::Module';
 
@@ -30,6 +35,11 @@ sub addUnauthRoute {
     return $self->_addRoute( 'addUnauthRoute', @_ );
 }
 
+sub addAuthRouteWithRedirect {
+    my $self = shift;
+    return $self->_addRoute( 'addAuthRouteWithRedirect', @_ );
+}
+
 sub _addRoute {
     my ( $self, $type, $word, $subName, $methods, $transform ) = @_;
     $transform //= sub {
@@ -38,13 +48,13 @@ sub _addRoute {
             return sub {
                 shift;
                 return $sub->( $self, @_ );
-              }
+            }
         }
         else {
             return sub {
                 shift;
                 return $self->$sub(@_);
-              }
+            }
         }
     };
     $self->p->$type( $word, $subName, $methods, $transform );
@@ -56,6 +66,43 @@ sub loadTemplate {
     return $self->p->loadTemplate(@_);
 }
 
+sub displayTemplate {
+    my ( $self, $req, $template, $params ) = @_;
+    $self->logger->debug("Return $template template");
+    $req->info( $self->loadTemplate( $req, $template, params => $params ) );
+    return PE_INFO;
+}
+
+sub createNotification {
+    my ( $self, $req, $uid, $date, $ref, $title, $msg ) = @_;
+    my $notifEngine = $self->p->loadedModules->{
+        'Lemonldap::NG::Portal::Plugins::Notifications'};
+
+    return PE_ERROR unless $notifEngine;
+
+    # Prepare notification
+    my $content =
+      $self->conf->{oldNotifFormat}
+      ? '<?xml version="1.0" encoding="UTF-8"?><root><notification uid="_uid_" date="_date_" reference="_ref_"><title>_title_</title><text>_msg_</text></notification></root>'
+      : '[{"uid":"_uid_","date":"_date_","title":"_title_","reference":"_ref_","text":"_msg_"}]';
+    $content =~ s/_uid_/$uid/;
+    $content =~ s/_date_/$date/;
+    $content =~ s/_ref_/$ref/;
+    $content =~ s/_title_/$title/;
+    $content =~ s/_msg_/$msg/;
+
+    if ( $notifEngine->module->notifObject->newNotification($content) ) {
+        $self->logger->debug("Notification $ref successfully created");
+        $self->userLogger->notice(
+            "Notification $ref / $date successfully created for $uid");
+        return PE_OK;
+    }
+    else {
+        $self->logger->debug("Notification $ref NOT created!");
+        return PE_ERROR;
+    }
+}
+
 1;
 __END__
 
@@ -146,6 +193,11 @@ Example:
       return $self->p->sendHtml($req, 'template', params => { WELLCOME => 1 });
   }
 
+If you want to get a "protected application" behavior, you can use
+B<addAuthRouteWithRedirect>. This methods calls B<addAuthRoute> with given
+arguments and build a "unAuth" route that build a redirection after
+authentication.
+
 =head3 Entry point in auth process
 
 A plugin which wants to be inserted in authentication process has to declare
@@ -175,6 +227,72 @@ process
 B<Note>: methods inserted so must return a PE_* constant. See
 Lemonldap::NG::Portal::Main::Constants.
 
+=head4 Advanced entry points
+
+These entry points are not stored in C<$req-E<gt>step> but launched on the fly:
+
+=over
+
+=item C<afterSub>: hash ref that give methods to call after given main method
+is called. Example:
+
+  use constant afterSub => {
+      getUser => 'mysub',
+  }
+  sub mysub {
+      my ( $self ,$req ) = @_;
+      # Do something
+      return PE_OK;
+  }
+
+=item C<aroundSub>: hash ref that give methods to call instead of given main
+method. Example:
+
+  use constant aroundSub => {
+      getUser => 'mysub',
+  };
+  sub mysub {
+      my ( $self, $sub, $req ) = @_;
+      # Do something before
+      my $ret = $sub->($req);
+      # Do something after
+      return $ret;
+  }
+
+Do not launch "getUser" but use the given C<$sub>. This permits multiple
+plugins to use "aroundSub" in the same time.
+
+=back
+
+=head1 LOGGING
+
+Logging is provided by $self->logger and $self->userLogger. The following rules
+must be applied:
+
+=over
+
+=item logger->debug: technical debugging messages
+
+=item logger->info: simple technical information
+
+=item logger->notice: technical information that could interest administrators
+
+=item logger->warn: technical warning
+
+=item logger->error: error that must be reported to administrator
+
+=item userLogger->info: simple information about user's action
+
+=item userLogger->notice: information that may be registered (auth success,...)
+
+=item userLogger->warn: bad action of a user (auth failure). Auth/Combination
+transform it to "info" when another authentication scheme is available
+
+=item userLogger->error: bad action of a user that must be reported, (even if
+another backend is available with Combination)
+
+=back
+
 =head1 SEE ALSO
 
 L<http://lemonldap-ng.org>
