@@ -9,10 +9,13 @@ use Lemonldap::NG::Portal::Main::Constants qw(
   PE_MALFORMEDUSER
 );
 
-our $VERSION = '2.0.3';
+our $VERSION = '2.0.6';
 
-extends 'Lemonldap::NG::Portal::Main::Plugin',
-  'Lemonldap::NG::Portal::Lib::_tokenRule';
+extends qw(
+  Lemonldap::NG::Portal::Main::Plugin
+  Lemonldap::NG::Portal::Lib::_tokenRule
+  Lemonldap::NG::Portal::Lib::OtherSessions
+);
 
 # INITIALIZATION
 
@@ -27,6 +30,7 @@ has ott => (
     }
 );
 has idRule => ( is => 'rw', default => sub { 1 } );
+has sorted => ( is => 'rw', default => sub { 0 } );
 
 sub hAttr {
     $_[0]->{conf}->{checkUserHiddenAttributes} . ' '
@@ -36,8 +40,8 @@ sub hAttr {
 sub init {
     my ($self) = @_;
     my $hd = $self->p->HANDLER;
-    $self->addAuthRoute( checkuser => 'check',   ['POST'] );
-    $self->addAuthRoute( checkuser => 'display', ['GET'] );
+    $self->addAuthRoute( checkuser => 'check', ['POST'] );
+    $self->addAuthRouteWithRedirect( checkuser => 'display', ['GET'] );
 
     # Parse identity rule
     $self->logger->debug(
@@ -50,7 +54,8 @@ sub init {
         return 0;
     }
     $self->idRule($rule);
-
+    $self->sorted( $self->conf->{impersonationRule}
+          || $self->conf->{contextSwitchingRule} );
     return 1;
 }
 
@@ -59,7 +64,9 @@ sub init {
 sub check {
     my ( $self, $req ) = @_;
     my ( $attrs, $array_attrs, $array_hdrs ) = ( {}, [], [] );
-    my $msg = my $auth = '';
+    my $msg       = my $auth = my $compute = '';
+    my $authLevel = $req->userData->{authenticationLevel};
+    my $authMode  = $req->userData->{_auth};
 
     # Check token
     if ( $self->ottRule->( $req, {} ) ) {
@@ -67,16 +74,19 @@ sub check {
         unless ($token) {
             $self->userLogger->warn('checkUser try without token');
             $msg   = PE_NOTOKEN;
-            $token = $self->ott->createToken( $req->userData );
+            $token = $self->ott->createToken();
         }
+
         unless ( $self->ott->getToken($token) ) {
             $self->userLogger->warn('checkUser try with expired/bad token');
             $msg   = PE_TOKENEXPIRED;
-            $token = $self->ott->createToken( $req->userData );
+            $token = $self->ott->createToken();
         }
+
         my $params = {
             PORTAL    => $self->conf->{portal},
             MAIN_LOGO => $self->conf->{portalMainLogo},
+            SKIN      => $self->p->getSkin($req),
             LANGS     => $self->conf->{showLanguages},
             MSG       => "PE$msg",
             ALERTE    => 'alert-warning',
@@ -105,31 +115,61 @@ sub check {
             params => {
                 PORTAL    => $self->conf->{portal},
                 MAIN_LOGO => $self->conf->{portalMainLogo},
+                SKIN      => $self->p->getSkin($req),
                 LANGS     => $self->conf->{showLanguages},
                 MSG       => 'PE' . PE_MALFORMEDUSER,
                 ALERTE    => 'alert-warning',
                 LOGIN     => '',
                 TOKEN     => (
                       $self->ottRule->( $req, {} )
-                    ? $self->ott->createToken( $req->userData )
+                    ? $self->ott->createToken()
                     : ''
                 )
             }
         );
     }
 
-    if ( $user eq $req->{user} or !$user ) {
-        $self->userLogger->notice("Retrieve session from Sessions database");
-        $self->userLogger->warn("Using spoofed SSO groups if exist!!!")
+    if ( !$user or $user eq $req->{user} ) {
+        $self->userLogger->info("checkUser requested for himself");
+        $self->userLogger->info("Using spoofed SSO groups if exist")
           if ( $self->conf->{impersonationRule} );
         $attrs = $req->userData;
+        $user  = $req->{user};
     }
     else {
-        $self->logger->debug("checkUser requested for $req->{user}");
-        $req->{user} = $user;
-        $self->userLogger->notice(
-            "Retrieve session from userDB and compute Groups & Macros");
-        $attrs = $self->_userDatas($req);
+        $self->userLogger->info("checkUser requested for $user");
+
+        # Try to retrieve session from sessions DB
+        $self->logger->debug('Try to retrieve session from DB...');
+        my $moduleOptions = $self->conf->{globalStorageOptions} || {};
+        $moduleOptions->{backend} = $self->conf->{globalStorage};
+
+        my $sessions    = {};
+        my $searchAttrs = $self->conf->{checkUserSearchAttributes}
+          || $self->conf->{whatToTrace};
+
+        foreach ( split /\s+/, $searchAttrs ) {
+            $self->logger->debug("Searching with: $_ = $user");
+            $sessions = $self->module->searchOn( $moduleOptions, $_, $user );
+            last if (keys %$sessions);
+        }
+
+        my $age = '1';
+        foreach my $id ( keys %$sessions ) {
+            my $session = $self->p->getApacheSession($id) or next;
+
+            if ( $session->{data}->{_utime} gt $age ) {
+                $attrs = $session->{data};
+                $age   = $session->{data}->{_utime};
+            }
+        }
+        unless ( defined $attrs->{_session_id} ) {
+            $req->{user} = $user;
+            $self->userLogger->info(
+                "No session found in DB. Compute userData...");
+            $attrs   = $self->_userData($req);
+            $compute = 1;
+        }
     }
 
     if ( $req->error ) {
@@ -138,7 +178,23 @@ sub check {
         $attrs       = {};
     }
     else {
-        $msg = 'checkUser';
+        $msg =
+          $self->{conf}->{impersonationMergeSSOgroups} eq 1
+          ? 'checkUserMerged'
+          : 'checkUser';
+        if ($compute) {
+            $msg                          = 'checkUserComputeSession';
+            $attrs->{authenticationLevel} = $authLevel;
+            $attrs->{_auth}               = $authMode;
+
+            if ( $self->conf->{impersonationRule} ) {
+                $self->logger->debug("Map real attributes...");
+                my %realAttrs = map {
+                    ( "$self->{conf}->{impersonationPrefix}$_" => $attrs->{$_} )
+                } keys %$attrs;
+                $attrs = { %$attrs, %realAttrs };
+            }
+        }
 
         # Create an array of hashes for template loop
         $self->logger->debug("Delete hidden or empty attributes");
@@ -170,23 +226,22 @@ sub check {
         $url = $self->_urlFormat($url);
 
         # User is allowed ?
-        $auth = $self->_authorization( $req, $url );
         $self->logger->debug(
-            "checkUser requested for user: $req->{user} and URL: $url");
+"checkUser requested for user: $attrs->{ $self->{conf}->{whatToTrace} } and URL: $url"
+        );
+        $auth = $self->_authorization( $req, $url, $attrs );
         if ( $auth >= 0 ) {
-
             $auth = $auth ? "allowed" : "forbidden";
-            $self->userLogger->notice( "checkUser -> $req->{user} is "
-                  . uc($auth)
-                  . " to access: $url" );
+            $self->logger->debug(
+                    "checkUser: $attrs->{ $self->{conf}->{whatToTrace} } is "
+                  . "$auth to access to $url" );
 
             # Return VirtualHost headers
-            $array_hdrs = $self->_headers( $req, $url );
+            $array_hdrs = $self->_headers( $req, $url, $attrs );
         }
         else {
             $auth = 'VHnotFound';
-            $self->userLogger->notice(
-                "checkUser -> URL: $url has no configuration");
+            $self->userLogger->info("checkUser: $url has no configuration");
         }
     }
 
@@ -198,14 +253,12 @@ sub check {
     my $params = {
         PORTAL    => $self->conf->{portal},
         MAIN_LOGO => $self->conf->{portalMainLogo},
+        SKIN      => $self->p->getSkin($req),
         LANGS     => $self->conf->{showLanguages},
         MSG       => $msg,
         ALERTE    => ( $msg eq 'checkUser' ? 'alert-info' : 'alert-warning' ),
-        LOGIN     => (
-              $self->p->checkXSSAttack( 'LOGIN', $req->{userData}->{uid} ) ? ""
-            : $req->{userData}->{uid}
-        ),
-        URL => (
+        LOGIN     => $user,
+        URL       => (
               $self->p->checkXSSAttack( 'URL', $url ) ? ""
             : $url
         ),
@@ -216,8 +269,7 @@ sub check {
         MACROS      => $array_attrs->[1],
         GROUPS      => $array_attrs->[0],
         TOKEN       => (
-              $self->ottRule->( $req, {} )
-            ? $self->ott->createToken( $req->userData )
+            $self->ottRule->( $req, {} ) ? $self->ott->createToken()
             : ''
         )
     };
@@ -231,8 +283,8 @@ sub display {
     my ( $self, $req ) = @_;
     my ( $attrs, $array_attrs ) = ( {}, [] );
 
-    $self->userLogger->notice("Retrieve session from Sessions database");
-    $self->userLogger->warn("Using spoofed SSO groups if exist!!!")
+    $self->logger->debug("Display current session data...");
+    $self->userLogger->info("Using spoofed SSO groups if exist")
       if ( $self->conf->{impersonationRule} );
     $attrs = $req->userData;
 
@@ -260,18 +312,24 @@ sub display {
 
     # Display form
     my $params = {
-        PORTAL     => $self->conf->{portal},
-        MAIN_LOGO  => $self->conf->{portalMainLogo},
-        LANGS      => $self->conf->{showLanguages},
-        MSG        => 'checkUser',
-        ALERTE     => 'alert-info',
-        LOGIN      => $req->{userData}->{uid},
+        PORTAL    => $self->conf->{portal},
+        MAIN_LOGO => $self->conf->{portalMainLogo},
+        SKIN      => $self->p->getSkin($req),
+        LANGS     => $self->conf->{showLanguages},
+        MSG       => (
+            $self->{conf}->{impersonationMergeSSOgroups} ? 'checkUserMerged'
+            : 'checkUser'
+        ),
+        ALERTE => (
+            $self->{conf}->{impersonationMergeSSOgroups} ? 'alert-warning'
+            : 'alert-info'
+        ),
+        LOGIN      => $req->{userData}->{ $self->conf->{whatToTrace} },
         ATTRIBUTES => $array_attrs->[2],
         MACROS     => $array_attrs->[1],
         GROUPS     => $array_attrs->[0],
         TOKEN      => (
-              $self->ottRule->( $req, {} )
-            ? $self->ott->createToken( $req->userData )
+            $self->ottRule->( $req, {} ) ? $self->ott->createToken()
             : ''
         )
     };
@@ -289,14 +347,13 @@ sub _urlFormat {
     $vhost =~ s/:\d+$//;
     $vhost .= $self->conf->{domain} unless ( $vhost =~ /\./ );
 
-    #$appuri ||= '/';
     return lc("$proto$vhost$port") . "$appuri";
 }
 
-sub _userDatas {
+sub _userData {
     my ( $self, $req ) = @_;
 
-    # Search user in database
+    # Compute session
     my $steps = [ 'getUser', 'setSessionInfo', 'setMacros', 'setGroups' ];
     $self->conf->{checkUserDisplayPersistentInfo}
       ? push @$steps, 'setPersistentSessionInfo', 'setLocalGroups'
@@ -313,6 +370,13 @@ sub _userDatas {
         return $req->error($error);
     }
 
+    unless ( defined $req->sessionInfo->{uid} ) {
+
+        # Avoid error with SAML, OIDC, etc...
+        $self->logger->debug("\"$req->{user}\" NOT found in userDB");
+        return $req->error(PE_BADCREDENTIALS);
+    }
+
     # Check identities rule
     unless ( $self->idRule->( $req, $req->sessionInfo ) ) {
         $self->userLogger->warn(
@@ -327,7 +391,7 @@ sub _userDatas {
 }
 
 sub _authorization {
-    my ( $self, $req, $uri ) = @_;
+    my ( $self, $req, $uri, $attrs ) = @_;
     my ( $vhost, $appuri ) = $uri =~ m#^https?://([^/]*)(.*)#;
     my $exist = 0;
 
@@ -340,23 +404,23 @@ sub _authorization {
         }
     }
 
-    $self->logger->debug("Return \"$req->{user}\" authorization");
+    $self->logger->debug(
+        "Return \"$attrs->{ $self->{conf}->{whatToTrace} }\" authorization");
     return $exist
-      ? $self->p->HANDLER->grant( $req, $req->{userData}, $appuri,
-        undef, $vhost )
+      ? $self->p->HANDLER->grant( $req, $attrs, $appuri, undef, $vhost )
       : -1;
 }
 
 sub _headers {
-    my ( $self, $req, $uri ) = @_;
+    my ( $self, $req, $uri, $attrs ) = @_;
     my ($vhost) = $uri =~ m#^https?://([^/]*).*#;
 
     $vhost =~ s/:\d+$//;
     $req->{env}->{HTTP_HOST} = $vhost;
     $self->p->HANDLER->headersInit( $self->{conf} );
-
-    $self->logger->debug("Return \"$req->{user}\" headers");
-    return $self->p->HANDLER->checkHeaders( $req, $req->{userData} );
+    $self->logger->debug(
+        "Return \"$attrs->{ $self->{conf}->{whatToTrace} }\" headers");
+    return $self->p->HANDLER->checkHeaders( $req, $attrs );
 }
 
 sub _splitAttributes {
@@ -366,13 +430,13 @@ sub _splitAttributes {
     $self->logger->debug("Dispatching attributes...");
     while (@$attrs) {
         my $element = shift @$attrs;
-        $self->logger->debug(
-            'Processing element: ' . Data::Dumper::Dumper($element) );
+        $self->logger->debug( "Processing element: $element->{key} => "
+              . ( $element->{value} // '' ) );
         my $ok = 0;
         if ( $element->{key} eq 'groups' ) {
             $self->logger->debug('Key "groups" found');
             my $separator = $self->{conf}->{multiValuesSeparator};
-            my @tmp = split /\Q$separator/, $element->{value};
+            my @tmp       = split /\Q$separator/, $element->{value};
             $grps = [ map { { value => $_ } } sort @tmp ];
             next;
         }
@@ -390,21 +454,22 @@ sub _splitAttributes {
     }
 
     # Sort real and spoofed attributes if required
-    if ( $self->conf->{impersonationRule} ) {
+    if ( $self->sorted ) {
         $self->logger->debug('Dispatching real and spoofed attributes...');
         my ( $realAttrs, $spoofedAttrs ) = ( [], [] );
         my $prefix = "$self->{conf}->{impersonationPrefix}";
         while (@$others) {
             my $element = shift @$others;
-            $self->logger->debug(
-                'Processing attribute: ' . Data::Dumper::Dumper($element) );
+            $self->logger->debug( "Processing attribute $element->{key} => "
+                  . ( $element->{value} // '' ) );
             if ( $element->{key} =~ /^$prefix.+$/ ) {
                 push @$realAttrs, $element;
                 $self->logger->debug(' -> Real attribute');
             }
             else {
                 push @$spoofedAttrs, $element;
-                $self->logger->debug(' -> Spoofed attribute');
+
+                #$self->logger->debug(' -> Spoofed attribute');
             }
         }
         @$others = ( @$spoofedAttrs, @$realAttrs );
